@@ -7,8 +7,20 @@
 #' @param y y geometry column
 #' @param z z geometry column
 #' @param m m geometry column
+#' @param keep logical indicating if the non-geometry and non-id columns should be kept.
+#' if TRUE you must supply the geometry and id columns, and only the first row of
+#' each geometry is kept. See Keeping Properties.
 #' @inheritSection sfc_point notes
 #'
+#' @section Keeping Properties:
+#'
+#' Setting \code{keep = TRUE} will retain the first row of any columns not specified as a
+#' coordinate (x, y, z, m) or an id (e.g., linestring_id, polygon_id) of the input \code{obj}.
+#'
+#' The \code{sf_*} functions assume the input \code{obj} is a long data.frame / matrix,
+#' where any properties are repeated down the table for the same geometry.
+#'
+#'
 #' @return \code{sf} object of POINT geometries
 #'
 #' @examples
@@ -24,10 +36,21 @@
 #' sf_point( obj = x, x = "x", y = "y" )
 #' sf_point( obj = x, x = "y", y = "x" )
 #'
+#' # keeping properties
+#' x$val <- letters[1:5]
+#' sf_point( x, x = "x", y = "y", keep = TRUE )
+#'
 #' @export
-sf_point <- function( obj, x = NULL, y = NULL, z = NULL, m = NULL ) {
+sf_point <- function(
+  obj,
+  x = NULL,
+  y = NULL,
+  z = NULL,
+  m = NULL,
+  keep = FALSE
+  ) {
   geometry_columns <- c(x,y,z,m)
-  rcpp_sf_point( obj, index_correct( geometry_columns ) )
+  return( rcpp_sf_point( obj, index_correct( geometry_columns ), keep ) )
 }
 
 #' sf MULTIPOINT
@@ -37,6 +60,7 @@ sf_point <- function( obj, x = NULL, y = NULL, z = NULL, m = NULL ) {
 #' @inheritParams sf_point
 #' @param obj sorted matrix or data.frame
 #' @inheritSection sfc_point notes
+#' @inheritSection sf_point Keeping Properties
 #' @param multipoint_id column of ids for multipoints
 #'
 #' @return \code{sf} object of MULTIPOINT geometries
@@ -52,19 +76,33 @@ sf_point <- function( obj, x = NULL, y = NULL, z = NULL, m = NULL ) {
 #' sf_multipoint( x, multipoint_id = "id", x = "x", y = "y")
 #'
 #' @export
-sf_multipoint <- function( obj, x = NULL, y = NULL, z = NULL, m = NULL, multipoint_id = NULL ) {
+sf_multipoint <- function(
+  obj,
+  x = NULL,
+  y = NULL,
+  z = NULL,
+  m = NULL,
+  multipoint_id = NULL,
+  keep = FALSE
+  ) {
   geometry_columns <- c(x,y,z,m)
-  res <- rcpp_sf_multipoint( obj, index_correct( geometry_columns ),  index_correct( multipoint_id ) )
+  res <- rcpp_sf_multipoint(
+    obj
+    , index_correct( geometry_columns )
+    , index_correct( multipoint_id )
+    , keep
+    )
   return( replace_id( res, multipoint_id ) )
 }
 
 
 #' sf LINESTRING
 #'
-#' constructs sf of MULTIPOINT objects
+#' constructs sf of LINESTRING objects
 #'
 #' @inheritParams sf_multipoint
 #' @inheritSection sfc_point notes
+#' @inheritSection sf_point Keeping Properties
 #' @param linestring_id column of ids for linestrings
 #'
 #' @return \code{sf} object of LINESTRING geometries
@@ -81,9 +119,22 @@ sf_multipoint <- function( obj, x = NULL, y = NULL, z = NULL, m = NULL, multipoi
 #' sf_linestring( x, linestring_id = "id", x = "x", y = "y")
 #'
 #' @export
-sf_linestring <- function( obj = NULL, x = NULL, y = NULL, z = NULL, m = NULL, linestring_id = NULL ) {
+sf_linestring <- function(
+  obj = NULL,
+  x = NULL,
+  y = NULL,
+  z = NULL,
+  m = NULL,
+  linestring_id = NULL,
+  keep = FALSE
+  ) {
   geometry_columns <- c(x,y,z,m)
-  res <- rcpp_sf_linestring( obj, index_correct( geometry_columns ),  index_correct( linestring_id ) )
+  res <- rcpp_sf_linestring(
+    obj
+    , index_correct( geometry_columns )
+    ,  index_correct( linestring_id )
+    , keep
+    )
   return( replace_id( res, linestring_id ) )
 }
 
@@ -97,6 +148,7 @@ sf_linestring <- function( obj = NULL, x = NULL, y = NULL, z = NULL, m = NULL, l
 #' @param linestring_id column of ids for linestrings (within multilinestrings)
 #'
 #' @inheritSection sfc_point notes
+#' @inheritSection sf_point Keeping Properties
 #'
 #' @return \code{sf} object of MULTILINESTRING geometries
 #'
@@ -129,9 +181,9 @@ sf_linestring <- function( obj = NULL, x = NULL, y = NULL, z = NULL, m = NULL, l
 #' sf_multilinestring( obj = df, x = "x", y = "y", z = "z")
 #' sf_multilinestring( obj = df, x = "x", y = "y", z = "z", m = "m")
 #'
-#' sf_multilinestring( obj = df, x = 2, y = 3)
-#' sf_multilinestring( obj = df, x = 2, y = 3, z = 4)
-#' sf_multilinestring( obj = df, x = 2, y = 3, z = 4, m = 5)
+#' sf_multilinestring( obj = df, x = 3, y = 4)
+#' sf_multilinestring( obj = df, x = 3, y = 4, z = 5)
+#' sf_multilinestring( obj = df, x = 3, y = 4, z = 5, m = 6 )
 #'
 #' sf_multilinestring( obj = df, multilinestring_id = "ml_id", linestring_id = "l_id" )
 #' sf_multilinestring( obj = df, multilinestring_id = 1, linestring_id = 2 )
@@ -139,9 +191,24 @@ sf_linestring <- function( obj = NULL, x = NULL, y = NULL, z = NULL, m = NULL, l
 #'
 #'
 #' @export
-sf_multilinestring <- function( obj = NULL, x = NULL, y = NULL, z = NULL, m = NULL, multilinestring_id = NULL, linestring_id = NULL ) {
+sf_multilinestring <- function(
+  obj = NULL,
+  x = NULL,
+  y = NULL,
+  z = NULL,
+  m = NULL,
+  multilinestring_id = NULL,
+  linestring_id = NULL,
+  keep = FALSE
+  ) {
   geometry_columns <- c(x,y,z,m)
-  res <- rcpp_sf_multilinestring( obj, index_correct( geometry_columns ), index_correct( multilinestring_id ), index_correct( linestring_id ) )
+  res <- rcpp_sf_multilinestring(
+    obj
+    , index_correct( geometry_columns )
+    , index_correct( multilinestring_id )
+    , index_correct( linestring_id )
+    , keep
+  )
   return( replace_id( res, multilinestring_id ) )
 }
 
@@ -157,6 +224,7 @@ sf_multilinestring <- function( obj = NULL, x = NULL, y = NULL, z = NULL, m = NU
 #' @param close logical indicating whether polygons should be closed. If \code{TRUE},
 #' all polygons will be checked and force closed if possible
 #' @inheritSection sfc_point notes
+#' @inheritSection sf_point Keeping Properties
 #'
 #' @return \code{sf} object of POLYGON geometries
 #'
@@ -199,9 +267,26 @@ sf_multilinestring <- function( obj = NULL, x = NULL, y = NULL, z = NULL, m = NU
 #'
 #'
 #' @export
-sf_polygon <- function( obj = NULL, x = NULL, y = NULL, z = NULL, m = NULL, polygon_id = NULL, linestring_id = NULL, close = TRUE ) {
+sf_polygon <- function(
+  obj = NULL,
+  x = NULL,
+  y = NULL,
+  z = NULL,
+  m = NULL,
+  polygon_id = NULL,
+  linestring_id = NULL,
+  close = TRUE,
+  keep = FALSE
+  ) {
   geometry_columns <- c(x,y,z,m)
-  res <- rcpp_sf_polygon( obj, index_correct( geometry_columns ), index_correct( polygon_id ), index_correct( linestring_id ), close )
+  res <- rcpp_sf_polygon(
+    obj
+    , index_correct( geometry_columns )
+    , index_correct( polygon_id )
+    , index_correct( linestring_id )
+    , close
+    , keep
+    )
   return( replace_id( res, polygon_id ) )
 }
 
@@ -214,6 +299,7 @@ sf_polygon <- function( obj = NULL, x = NULL, y = NULL, z = NULL, m = NULL, poly
 #' @inheritParams sf_polygon
 #' @param multipolygon_id column of ids for multipolygons
 #' @inheritSection sfc_point notes
+#' @inheritSection sf_point Keeping Properties
 #'
 #' @return \code{sf} object of MULTIPOLYGON geometries
 #'
@@ -266,7 +352,7 @@ sf_polygon <- function( obj = NULL, x = NULL, y = NULL, z = NULL, m = NULL, poly
 #' sf_multipolygon( df, multipolygon_id = "id1", polygon_id = "id2" )
 #' sf_multipolygon( df, polygon_id = "id1", linestring_id = "id2" )
 #' sf_multipolygon( df, x = "x", y = "y", polygon_id = "id1")
-#' sf_multipolygon( df, x = "x", y = "y", polygon_id = "id1", linestring_id =)
+#' sf_multipolygon( df, x = "x", y = "y", polygon_id = "id1", linestring_id = "id2")
 #' sf_multipolygon( df, x = "x", y = "y", linestring_id = "id1")
 #' sf_multipolygon( df, x = "x", y = "y", linestring_id = "id2")
 #'
@@ -280,9 +366,28 @@ sf_polygon <- function( obj = NULL, x = NULL, y = NULL, z = NULL, m = NULL, poly
 #' sf_multipolygon( df, x = "x", y = "y", polygon_id = "id1")
 #'
 #' @export
-sf_multipolygon <- function( obj = NULL, x = NULL, y = NULL, z = NULL, m = NULL, multipolygon_id = NULL, polygon_id = NULL, linestring_id = NULL, close = TRUE ) {
+sf_multipolygon <- function(
+  obj = NULL,
+  x = NULL,
+  y = NULL,
+  z = NULL,
+  m = NULL,
+  multipolygon_id = NULL,
+  polygon_id = NULL,
+  linestring_id = NULL,
+  close = TRUE,
+  keep = FALSE
+  ) {
   geometry_columns <- c(x,y,z,m)
-  res <- rcpp_sf_multipolygon( obj, index_correct( geometry_columns ), index_correct( multipolygon_id ), index_correct( polygon_id ), index_correct( linestring_id ), close )
+  res <- rcpp_sf_multipolygon(
+    obj
+    , index_correct( geometry_columns )
+    , index_correct( multipolygon_id )
+    , index_correct( polygon_id )
+    , index_correct( linestring_id )
+    , close
+    , keep
+    )
   return( replace_id( res, multipolygon_id ) )
 }
 
