@@ -439,12 +439,10 @@ inline SEXP sfc_multipoint(
   return Rcpp::List::create();
 }
 
-
-
 inline SEXP sfc_multipoint(
-    Rcpp::IntegerMatrix& im,
-    Rcpp::IntegerVector& geometry_cols,
-    Rcpp::IntegerVector& line_ids
+  Rcpp::IntegerMatrix& im,
+  Rcpp::IntegerVector& geometry_cols,
+  Rcpp::IntegerMatrix& line_positions
 ) {
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -455,10 +453,8 @@ inline SEXP sfc_multipoint(
   R_xlen_t n_col = im.ncol();
   sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, im, geometry_cols );
 
-  Rcpp::IntegerVector unique_ids = Rcpp::sort_unique( line_ids );
-  Rcpp::IntegerMatrix line_positions = sfheaders::utils::id_positions( line_ids, unique_ids );
 
-  R_xlen_t n_lines = unique_ids.length();
+  R_xlen_t n_lines = line_positions.nrow();
 
   Rcpp::List sfc( n_lines );
 
@@ -483,11 +479,19 @@ inline SEXP sfc_multipoint(
   return sfc;
 }
 
+inline SEXP sfc_multipoint(
+    Rcpp::IntegerMatrix& im,
+    Rcpp::IntegerVector& geometry_cols,
+    Rcpp::IntegerVector& line_ids
+) {
+  Rcpp::IntegerMatrix line_positions = sfheaders::utils::id_positions( line_ids );
+  return sfc_multipoint( im, geometry_cols, line_positions );
+}
 
 inline SEXP sfc_multipoint(
-    Rcpp::NumericMatrix& nm,
-    Rcpp::IntegerVector& geometry_cols,
-    Rcpp::NumericVector& line_ids
+  Rcpp::NumericMatrix& nm,
+  Rcpp::IntegerVector& geometry_cols,
+  Rcpp::IntegerMatrix& line_positions
 ) {
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -498,10 +502,8 @@ inline SEXP sfc_multipoint(
   R_xlen_t n_col = nm.ncol();
   sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, nm, geometry_cols );
 
-  Rcpp::NumericVector unique_ids = Rcpp::sort_unique( line_ids );
-  Rcpp::IntegerMatrix line_positions = sfheaders::utils::id_positions( line_ids, unique_ids );
 
-  R_xlen_t n_lines = unique_ids.length();
+  R_xlen_t n_lines = line_positions.nrow();
 
   Rcpp::List sfc( n_lines );
 
@@ -527,9 +529,18 @@ inline SEXP sfc_multipoint(
 }
 
 inline SEXP sfc_multipoint(
-    Rcpp::DataFrame& df,
-    Rcpp::StringVector& geometry_cols,
-    SEXP& line_ids
+    Rcpp::NumericMatrix& nm,
+    Rcpp::IntegerVector& geometry_cols,
+    Rcpp::NumericVector& line_ids
+) {
+  Rcpp::IntegerMatrix line_positions = sfheaders::utils::id_positions( line_ids );
+  return sfc_multipoint( nm, geometry_cols, line_positions );
+}
+
+inline SEXP sfc_multipoint(
+  Rcpp::DataFrame& df,
+  Rcpp::StringVector& geometry_cols,
+  Rcpp::IntegerMatrix& line_positions
 ) {
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -540,10 +551,8 @@ inline SEXP sfc_multipoint(
   R_xlen_t n_col = df.ncol();
   sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
 
-  SEXP unique_ids = sfheaders::utils::get_sexp_unique( line_ids );
-  Rcpp::IntegerMatrix line_positions = sfheaders::utils::id_positions( line_ids, unique_ids );
 
-  R_xlen_t n_lines = sfheaders::utils::get_sexp_length( unique_ids );
+  R_xlen_t n_lines = line_positions.nrow();
 
   Rcpp::List sfc( n_lines );
 
@@ -568,11 +577,19 @@ inline SEXP sfc_multipoint(
   return sfc;
 }
 
-
 inline SEXP sfc_multipoint(
     Rcpp::DataFrame& df,
-    Rcpp::IntegerVector& geometry_cols,
+    Rcpp::StringVector& geometry_cols,
     SEXP& line_ids
+) {
+  Rcpp::IntegerMatrix line_positions = sfheaders::utils::id_positions( line_ids );
+  return sfc_multipoint( df, geometry_cols, line_positions );
+}
+
+inline SEXP sfc_multipoint(
+  Rcpp::DataFrame& df,
+  Rcpp::IntegerVector& geometry_cols,
+  Rcpp::IntegerMatrix& line_positions
 ) {
   Rcpp::NumericVector bbox = sfheaders::bbox::start_bbox();
   Rcpp::NumericVector z_range = sfheaders::zm::start_z_range();
@@ -583,10 +600,8 @@ inline SEXP sfc_multipoint(
   R_xlen_t n_col = df.ncol();
   sfheaders::zm::calculate_zm_ranges( n_col, z_range, m_range, df, geometry_cols );
 
-  SEXP unique_ids = sfheaders::utils::get_sexp_unique( line_ids );
-  Rcpp::IntegerMatrix line_positions = sfheaders::utils::id_positions( line_ids, unique_ids );
 
-  R_xlen_t n_lines = sfheaders::utils::get_sexp_length( unique_ids );
+  R_xlen_t n_lines = line_positions.nrow();
 
   Rcpp::List sfc( n_lines );
 
@@ -610,6 +625,14 @@ inline SEXP sfc_multipoint(
   sfheaders::sfc::make_sfc( sfc, sfheaders::sfc::SFC_MULTIPOINT, bbox, z_range, m_range );
   return sfc;
 }
+inline SEXP sfc_multipoint(
+    Rcpp::DataFrame& df,
+    Rcpp::IntegerVector& geometry_cols,
+    SEXP& line_ids
+) {
+  Rcpp::IntegerMatrix line_positions = sfheaders::utils::id_positions( line_ids );
+  return sfc_multipoint( df, geometry_cols, line_positions );
+}
 
 inline SEXP sfc_multipoint(
     Rcpp::IntegerMatrix& im,
